@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014,2015,2016,2017,2018
+;;  Copyright (C) 2014,2015,2016,2017,2018,2019
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -28,10 +28,14 @@
   #:re-export (artanis-version)
   #:export (*handlers-table*
             *conf-hash-table*
+            *http-options-table*
             *conn-pool*
             *before-response-hook*
             *after-request-hook*
             *before-run-hook*
+            *after-websocket-handshake-hook*
+            *DB-conn-init-hook*
+            *when-sigint-hook*
             *sql-mapping-lookup-table*
             *artanis-entry*
             %current-toplevel
@@ -49,7 +53,7 @@
             current-session-backend
             protocol-add!
             lookup-protocol
-            must-close-connection?
+            half-closed?
             %os-kernel
             %os-distro
             %kernel-version
@@ -57,7 +61,17 @@
             kernel-version>=?
             linux-version>=?
             is-init-server-run?
-            current-encoder))
+            current-encoder
+            *lpc-instance-pool*
+            out-of-task-prompt?
+            resources-collecting?
+            oh-define!
+            oh-set!
+            oh-ref
+            is-mmapped-file?
+            register-mmapped-file!
+            http-status
+            get-syspage-handler))
 
 ;; WARNING:
 ;; For concurrency in green-thread, all these stuffs should be immutable
@@ -73,6 +87,10 @@
 (define *handlers-table* (make-hash-table))
 (define *conf-hash-table* (make-hash-table))
 
+;; table structure:
+;; ("<http-url-rules>" (<methods-list))
+(define *http-options-table* (make-hash-table))
+
 ;; NOTE: The init queue size equals to (server wqlen).
 ;;       If all the available DB conn were blocked, a new DB conn will be
 ;;       created, and never closed but just recycled by *conn-pool*.
@@ -81,6 +99,9 @@
 (define *before-response-hook* (make-hook 2))
 (define *after-request-hook* (make-hook 2))
 (define *before-run-hook* (make-hook))
+(define *DB-conn-init-hook* (make-hook 1))
+(define *after-websocket-handshake-hook* (make-hook 2))
+(define *when-sigint-hook* (make-hook))
 
 ;; TODO: I don't have much time for it but it should be RB-Tree in the future
 (define *sql-mapping-lookup-table* (make-hash-table))
@@ -141,7 +162,7 @@
 (define (lookup-protocol name)
   (hashq-ref *proto-table* name))
 
-(define must-close-connection? (make-parameter #f))
+(define half-closed? (make-parameter #f))
 
 (define (%os-kernel)
   (vector-ref (uname) 0))
@@ -197,3 +218,30 @@
 (define is-init-server-run? #f)
 
 (define current-encoder (make-parameter identity))
+
+(define *lpc-instance-pool* #f)
+
+(define out-of-task-prompt? (make-parameter #f))
+
+(define resources-collecting? (make-parameter #f))
+
+(define *options-meta-handler-table* '())
+(define (oh-define! lst) (set! *options-meta-handler-table* lst))
+(define (oh-set! k h)
+  (set! *options-meta-handler-table*
+    (assq-set! *options-meta-handler-table* k h)))
+(define (oh-ref o)
+  (assq-ref *options-meta-handler-table* o))
+
+;; (client-fd . mmapped-bv)
+(define *mmapped-bv-from-file* (make-hash-table))
+(define (register-mmapped-file! fd bv)
+  (hashv-set! *mmapped-bv-from-file* fd bv))
+(define (is-mmapped-file? fd)
+  (hashv-ref *mmapped-bv-from-file* fd #f))
+
+(define *sys-page-handlers* (make-hash-table))
+(define (http-status status thunk)
+  (hashv-set! *sys-page-handlers* status thunk))
+(define (get-syspage-handler status)
+  (hashv-ref *sys-page-handlers* status #f))
